@@ -406,12 +406,25 @@ class Stylesheet:
         )
         self.replace_rules(node, node_rules, animate=animate)
 
-        node._component_styles.clear()
-        for component in node._get_component_classes():
-            virtual_node = DOMNode(classes=component)
-            virtual_node._attach(node)
-            self.apply(virtual_node, animate=False)
-            node._component_styles[component] = virtual_node.styles
+        component_classes = node._get_component_classes()
+        if component_classes:
+            # Create virtual nodes that exist to extract styles
+            refresh_node = False
+            old_component_styles = node._component_styles.copy()
+            node._component_styles.clear()
+            for component in sorted(component_classes):
+                virtual_node = DOMNode(classes=component)
+                virtual_node._attach(node)
+                self.apply(virtual_node, animate=False)
+                if (
+                    not refresh_node
+                    and old_component_styles.get(component) != virtual_node.styles
+                ):
+                    # If the styles have changed we want to refresh the node
+                    refresh_node = True
+                node._component_styles[component] = virtual_node.styles
+            if refresh_node:
+                node.refresh()
 
     @classmethod
     def replace_rules(
@@ -487,7 +500,7 @@ class Stylesheet:
             for key in modified_rule_keys:
                 setattr(base_styles, key, get_rule(key))
 
-        node.post_message_no_wait(messages.StylesUpdated(sender=node))
+        node.notify_style_update()
 
     def update(self, root: DOMNode, animate: bool = False) -> None:
         """Update styles on node and its children.
