@@ -69,14 +69,18 @@ class ScrollView(Widget):
         return self.virtual_size.height
 
     def _size_updated(
-        self, size: Size, virtual_size: Size, container_size: Size
-    ) -> None:
+        self, size: Size, virtual_size: Size, container_size: Size, layout: bool = True
+    ) -> bool:
         """Called when size is updated.
 
         Args:
             size: New size.
             virtual_size: New virtual size.
             container_size: New container size.
+            layout: Perform layout if required.
+
+        Returns:
+            True if anything changed, or False if nothing changed.
         """
         if self._size != size or container_size != container_size:
             self.refresh()
@@ -90,6 +94,9 @@ class ScrollView(Widget):
             self._container_size = size - self.styles.gutter.totals
             self._scroll_update(virtual_size)
             self.scroll_to(self.scroll_x, self.scroll_y, animate=False)
+            return True
+        else:
+            return False
 
     def render(self) -> RenderableType:
         """Render the scrollable region (if `render_lines` is not implemented).
