@@ -1,15 +1,16 @@
 from __future__ import annotations
-from importlib import import_module
+
 import typing
+from importlib import import_module
 
 from ..case import camel_to_snake
 
-# ⚠️For any new built-in Widget we create, not only do we have to import them here and add them to `__all__`,
+# For any new built-in Widget we create, not only do we have to import them here and add them to `__all__`,
 # but also to the `__init__.pyi` file in this same folder - otherwise text editors and type checkers won't
 # be able to "see" them.
 if typing.TYPE_CHECKING:
+    from ..widget import Widget
     from ._button import Button
-    from ._checkbox import Checkbox
     from ._data_table import DataTable
     from ._directory_tree import DirectoryTree
     from ._footer import Footer
@@ -18,19 +19,18 @@ if typing.TYPE_CHECKING:
     from ._label import Label
     from ._list_item import ListItem
     from ._list_view import ListView
+    from ._markdown import Markdown, MarkdownViewer
     from ._placeholder import Placeholder
     from ._pretty import Pretty
     from ._static import Static
+    from ._switch import Switch
     from ._text_log import TextLog
     from ._tree import Tree
-    from ._tree_node import TreeNode
     from ._welcome import Welcome
-    from ..widget import Widget
 
 
 __all__ = [
     "Button",
-    "Checkbox",
     "DataTable",
     "DirectoryTree",
     "Footer",
@@ -39,12 +39,14 @@ __all__ = [
     "Label",
     "ListItem",
     "ListView",
+    "Markdown",
+    "MarkdownViewer",
     "Placeholder",
     "Pretty",
     "Static",
+    "Switch",
     "TextLog",
     "Tree",
-    "TreeNode",
     "Welcome",
 ]
 
